// table.cpp : implementation of the CTable class
//

#include "stdafx.h"
#include "table.h"
#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


void SetGlassMaterial()
{
	CMaterial::SetMaterial(EMERALD,GL_FRONT);
	CMaterial::SetMaterial(RED_PLASTIC,GL_BACK);
}


void SetTallerkenMaterial()
{
	CMaterial::SetMaterial(WARM_WHITE,GL_BACK);
	CMaterial::SetMaterial(WHITE_RUBBER,GL_FRONT);
}

void SetBordMaterial()
{
	CMaterial::SetMaterial(YELLOW_RUBBER,GL_FRONT_AND_BACK);
}

void SetLysestakeMaterial()
{
	CMaterial::SetMaterial(GREEN_PLASTIC,GL_FRONT_AND_BACK);
}

// precision in drawing
#define qual 15
/////////////////////////////////////////////////////////////////////////////
// CTable

IMPLEMENT_DYNCREATE(CTable, CObject)


/////////////////////////////////////////////////////////////////////////////
// CTable construction/destruction
CTable::CTable()
{
	glpQ = gluNewQuadric();
	displaymode=GL_RENDER;
}


CTable::~CTable()
{
	gluDeleteQuadric(glpQ);
}
/////////////////////////////////////////////////////////////////////////////
// CTablev diagnostics

#ifdef _DEBUG
void CTable::AssertValid() const
{
	CObject::AssertValid();
}

void CTable::Dump(CDumpContext& dc) const
{
	CObject::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
void CTable::Glass(GLfloat h1,GLfloat h2,GLfloat h3,
				   GLfloat b,GLfloat v1,GLfloat v3)
{
	// draw a glass with pos z-axes in centerline
	// standing on xy-plane
	
	glPushMatrix();
	
	if(h3>0.0f) // bottom part
	{	
		// must find bottom radius,x,from b and v3
		// tg(v3)=x/h3, x=h3*tg(v3)
		GLfloat x=h3*(GLfloat)tan((double)v3*GtR);
		gluCylinder(glpQ,b/2.0f+x,b/2.0f,h3,qual,qual);
	}

	if(h2>0.0f) // middle part
	{
		if(h3>0.0f)
			glTranslatef(0.0f,0.0f,h3+0.001f);
		gluDisk(glpQ,0.0f,b/2.0f,qual,qual);
		gluCylinder(glpQ,b/2.0f,b/2.0f,h2,qual,qual);
	}

	if(h1>0.0f) // top part
	{
		if(h2>0.0f)
			glTranslatef(0.0f,0.0f,h2+0.001f);
		// must find top radius,x,from b and v1
		// tg(v1)=x/h1, x=h1*tg(v1)
		GLfloat x=h1*(GLfloat)tan((double)v1*GtR);
		gluDisk(glpQ,0.0f,b/2.0f,qual,qual);
		gluCylinder(glpQ,b/2.0f,b/2.0f+x,h1,qual,qual);
		
	}
	glPopMatrix();
}

BOOL CTable::Bord(GLfloat r,GLint antall)
{
	// draw a circular table with antall plates and glasses
	if(displaymode==GL_RENDER) // to speed up selection
	{
		// make the table unconditionally
		SetBordMaterial();
		// the table is tt cm thick
		GLfloat tt=2.0f;
		glPushMatrix();
		glTranslatef(0.0f,0.0f,-tt);
		gluCylinder(glpQ,r,r,tt,qual,qual);
		gluDisk(glpQ,0.0f,r,qual,qual);
		glPopMatrix();
		gluDisk(glpQ,0.0f,r,qual,qual);
	}

	// then the candelabers	and the lightbulbs
	SetLysestakeMaterial();
	glPushMatrix();
	glTranslatef(r/3.0f,0.0f,0.0f);
	if(displaymode==GL_SELECT)
		glLoadName(1);
	Glass(20.0f,-1.0f,1.0f,2.0f,0.0f,60.0f);
	if(glIsEnabled(GL_LIGHT1))
		DrawLight(GL_LIGHT1);
	glPopMatrix();

	SetLysestakeMaterial();
	glPushMatrix();
	glTranslatef(-r/3.0f,0.0f,0.0f);
	if(displaymode==GL_SELECT)
		glLoadName(2);
	Glass(20.0f,-1.0f,1.0f,2.0f,0.0f,60.0f);
	if(glIsEnabled(GL_LIGHT2))
		DrawLight(GL_LIGHT2);
	glPopMatrix();

	if(displaymode==GL_SELECT) // to speed up selection
		return TRUE;

	// make some decisions on layout
	// glasses has (max) radius		gr=3.5 cm
	// plates has radius of approx	pr=10.0 cm
	// Center for glasses and plates are at a radius sr=r-pr-1.0
	// Distance from center plate to center glass is d=gr+pr+1.0
	GLfloat gr=3.5f;
	GLfloat pr=10.0f;
	GLfloat sr=r-pr-1.0f;
	GLfloat d=gr+pr+1.0f;
	// each couvert has at its disposal dv =360/antall
	GLfloat dv=360.0f/(GLfloat)antall;
	if(antall>3)
	{
		// each couvert needs roughly nv, 
		// where  tg(nv)=(pr+gr+d+1.0)/sr
		// so if tg(dv)<(pr+gr+d+1.0)/sr we cannot make the table
		if( tan((double)dv*GtR) <  ((pr+gr+d+1.0f)/sr) )
			return FALSE;
	}

	// ok we assume we can do it
	// take the couverts in turn
	glPushMatrix();
	glTranslatef(0.0f,0.0f,0.01f); // just above table
	GLfloat v=0.0f;
	for(int ix=0;ix<antall;ix++)
	{
		glPushMatrix();
		// the geometric reasoning is:
		// 1 take the glass out in x on plates side
		// 2 move both out sr, in y direction
		// 3 rotate both v around z
		glRotatef(v,0.0f,0.0f,1.0f);
		glTranslatef(0.0f,sr,0.0f);
		SetTallerkenMaterial();
		Glass(2.0f,-1.0f,-1.0f,17.0f,30.0f,0.0f); //plate
		glTranslatef(d,0.0f,0.0f);
		SetGlassMaterial();
		Glass(5.0f,3.0f,2.0f,1.0f,30.0f,60.0f); //glass
		glPopMatrix();
		v+=dv;
	}
	glPopMatrix();
	return TRUE;
}

void CTable::DrawLight(GLenum L)
{
	// we are positioned at the bottom of the candelaber
	// set lightposition
	CMaterial::SetMaterial(BRIGHT_WHITE);
	// draw light as a bulb
	glTranslatef(0.0f,0.0f,22.0f);		// over candelaber
	GLfloat p[]={0.0f,0.0f,0.0f,1.0f};	// directional
	GLfloat spdirection[] = {0.0f,0.0f,-1.0f};// down on table
	GLfloat	spotangle=70.0f;
	glLightfv(L,GL_POSITION,p);
	glLightfv(L,GL_SPOT_DIRECTION,spdirection);
	glLightf (L,GL_SPOT_CUTOFF,spotangle);
	glEnable(L);
	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT,GL_EMISSION);
	glColor3f(1.0f,1.0f,1.0f);
	gluSphere(glpQ,2.0f,10,10);		// the bulb
	glColor3f(0.0f,0.0f,0.0f);
	glDisable(GL_COLOR_MATERIAL);
}

void CTable::SetDisplayMode(GLenum mode)
{
	displaymode=mode;
}

